/******************************************************************************
 * Copyright(c) 2008 - 2010 Realtek Corporation. All rights reserved.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
 *
 * The full GNU General Public License is included in this distribution in the
 * file called LICENSE.
 *
 * Contact Information:
 * Jerry chuang <wlanfae@realtek.com>
******************************************************************************/

#ifdef LED

#include "../ieee80211/ieee80211.h"
#include "r8180_hw.h"
#include "r8180.h"
#include "r8180_93cx6.h" 
#include "r8185_led.h"


#define LED_BLINK_NORMAL_INTERVAL	100
#define LED_BLINK_SLOWLY_INTERVAL	200

#define LED_CM2_BLINK_ON_INTERVAL		250
#define LED_CM2_BLINK_OFF_INTERVAL		4750

#define LED_CM3_BLINK_INTERVAL			1500



static void
InitLed8185(
	struct net_device 		*dev, 
	PLED_8185			pLed,
	LED_PIN_8185			LedPin
	);

static void
DeInitLed8185(
	PLED_8185			pLed
	);

static void
BlinkTimerCallback(
	unsigned long		data
	);

static void
SwLedOn(
	struct net_device 		*dev, 
	PLED_8185			pLed
	);

static void
SwLedOff(
	struct net_device 		*dev, 
	PLED_8185			pLed
	);

static void
SwLedControlMode0(
	struct net_device 		*dev,
	LED_CTL_MODE		LedAction
	);

static void
SwLedBlink(
	PLED_8185			pLed
	);


void
InitLed8185(
	struct net_device 	*dev, 
	PLED_8185		pLed,
	LED_PIN_8185		LedPin
	)
{
	struct r8180_priv *priv = ieee80211_priv(dev);

	pLed->pAdapter = dev;

	pLed->LedPin = LedPin;

	pLed->CurrLedState = LED_OFF;
	pLed->bLedOn = false;

	pLed->bLedBlinkInProgress = false;
	pLed->BlinkTimes = 0;
	pLed->BlinkingLedState = LED_OFF;

	init_timer(&(pLed->BlinkTimer));
	pLed->BlinkTimer.data = (unsigned long)dev;
	pLed->BlinkTimer.function = BlinkTimerCallback;	

	priv->pLed = pLed;

	{
	u16 usValue = 0;
	
	usValue = eprom_read(dev, EEPROM_SW_REVD_OFFSET>>1);
	priv->InterfaceSel = (INTERFACE_SELECT_8185)( ( usValue & EEPROM_BUS_MASK )>> 6 );

	if(priv->InterfaceSel == INTF_SEL1_MINICARD)
		priv->LedStrategy = SW_LED_MODE0;
	else
		priv->LedStrategy = SW_LED_MODE0;
	}
}


void
DeInitLed8185(
	PLED_8185			pLed
	)
{

	del_timer_sync(&(pLed->BlinkTimer));
	
	pLed->bLedBlinkInProgress = false;

}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
void SwLedWorkItemCallback(struct work_struct *work)
{
	struct r8180_priv *priv = container_of(work, struct r8180_priv, SwLedWorkItem);
#else
void  SwLedWorkItemCallback(void *	Context)
{
	struct net_device *dev = (struct net_device *)Context;
	struct r8180_priv *priv = ieee80211_priv(dev);
#endif

	PLED_8185	pLed = priv->pLed;

	SwLedBlink(pLed);
}

void
BlinkTimerCallback(
	unsigned long		data
	)
{
	struct net_device *dev = (struct net_device *)data;
	struct r8180_priv *priv = ieee80211_priv(dev);
	
	
	switch(priv->LedStrategy)
	{
	case SW_LED_MODE0:
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
		schedule_work(&(priv->SwLedWorkItem));
#else	
		schedule_task(&(priv->SwLedWorkItem));
#endif	
		break;
	
	default:
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
		schedule_work(&(priv->SwLedWorkItem));
#else	
		schedule_task(&(priv->SwLedWorkItem));
#endif	
		break;
	}	
}


void
SwLedOn(
	struct net_device 		*dev, 
	PLED_8185			pLed
)
{


	switch(pLed->LedPin)
	{
	case LED_PIN_LED0:
		write_nic_byte(dev, LED_CONTROL, 0x02);
		break;
       
	default:
		break;
	}

	pLed->bLedOn = true;
}



void
SwLedOff(
	struct net_device 		*dev, 
	PLED_8185			pLed
)
{

	switch(pLed->LedPin)
	{
	case LED_PIN_LED0:
		write_nic_byte(dev, LED_CONTROL, 0x03); 
		break;

	default:
		break;
	}

	pLed->bLedOn = false;
}




void
InitSwLeds(
	struct net_device *dev
	)
{
	struct r8180_priv *priv = ieee80211_priv(dev);

	InitLed8185(dev,&(priv->SwLed0), LED_PIN_LED0);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20))
	INIT_WORK(&priv->SwLedWorkItem, (void*) SwLedWorkItemCallback);
#elif (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
	INIT_WORK(&priv->SwLedWorkItem, (void*) SwLedWorkItemCallback,dev);
#else
	tq_init(&priv->SwLedWorkItem, (void*) SwLedWorkItemCallback,dev);
#endif
}


void
DeInitSwLeds(
	struct net_device *dev
	)
{
	struct r8180_priv *priv = ieee80211_priv(dev);

	DeInitLed8185( &(priv->SwLed0) );
}

void
LedControl8185(
	struct net_device 		*dev,
	LED_CTL_MODE		LedAction
	)
{

	struct r8180_priv *priv = ieee80211_priv(dev);


	if(	priv->eRFPowerState != eRfOn && 
		(LedAction == LED_CTL_TX || LedAction == LED_CTL_RX || 
		 LedAction == LED_CTL_SITE_SURVEY || 
		 LedAction == LED_CTL_LINK || 
		 LedAction == LED_CTL_NO_LINK) )
	{
		return;
	}

	switch(priv->LedStrategy)
	{
	case SW_LED_MODE0:
		SwLedControlMode0(dev, LedAction);
		break;

	default:
		break;
	}
}



void
SwLedControlMode0(
	struct net_device 		*dev,
	LED_CTL_MODE		LedAction
	)
{
	struct r8180_priv *priv = ieee80211_priv(dev);
	PLED_8185 pLed = &(priv->SwLed0);

	switch(LedAction)
	{
	case LED_CTL_TX:
	case LED_CTL_RX:
		if( pLed->bLedBlinkInProgress == false )
		{
			pLed->bLedBlinkInProgress = true;

			pLed->CurrLedState = LED_BLINK_NORMAL;
			pLed->BlinkTimes = 2;

			if( pLed->bLedOn )
				pLed->BlinkingLedState = LED_OFF; 
			else
				pLed->BlinkingLedState = LED_ON; 
			mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_BLINK_NORMAL_INTERVAL));
		}
		break;

	case LED_CTL_SITE_SURVEY:
		if( pLed->bLedBlinkInProgress == false )
		{
			pLed->bLedBlinkInProgress = true;
			
			if(((priv->ieee80211->state == IEEE80211_ASSOCIATING) || 
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATED) ||
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_RETRY) ||
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATING)) || 
		        (priv->ieee80211->iw_mode == IW_MODE_ADHOC))
			{
				pLed->CurrLedState = LED_SCAN_BLINK;
				pLed->BlinkTimes = 4;
			}
			else
			{
				pLed->CurrLedState = LED_NO_LINK_BLINK;
				pLed->BlinkTimes = 24;
			}

			if( pLed->bLedOn )
			{
				pLed->BlinkingLedState = LED_OFF; 
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_ON_INTERVAL));
			}
			else
			{
				pLed->BlinkingLedState = LED_ON; 
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_OFF_INTERVAL));
			}
		}
		else
		{
			if(pLed->CurrLedState != LED_NO_LINK_BLINK)
			{
				if(((priv->ieee80211->state == IEEE80211_ASSOCIATING) || 
		    		(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATED) ||
		    		(priv->ieee80211->state == IEEE80211_ASSOCIATING_RETRY) ||
		    		(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATING)) || 
		        	(priv->ieee80211->iw_mode == IW_MODE_ADHOC))
				{
					pLed->CurrLedState = LED_SCAN_BLINK;
				}
				else
				{
					pLed->CurrLedState = LED_NO_LINK_BLINK;
				}
			}
		}
		break;

	case LED_CTL_NO_LINK:
		if( pLed->bLedBlinkInProgress == false )
		{
			pLed->bLedBlinkInProgress = true;

			pLed->CurrLedState = LED_NO_LINK_BLINK;
			pLed->BlinkTimes = 24;

			if( pLed->bLedOn )
			{
				pLed->BlinkingLedState = LED_OFF; 
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_ON_INTERVAL));
			}
			else
			{
				pLed->BlinkingLedState = LED_ON; 
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_OFF_INTERVAL));
			}
		}
		else
		{
			pLed->CurrLedState = LED_NO_LINK_BLINK;
		}
		break;

	case LED_CTL_LINK:
		pLed->CurrLedState = LED_ON;
		if( pLed->bLedBlinkInProgress == false )
		{
			SwLedOn(dev, pLed);
		}
		break;

	case LED_CTL_POWER_OFF:
		pLed->CurrLedState = LED_OFF;
		if(pLed->bLedBlinkInProgress)
		{
			del_timer_sync(&(pLed->BlinkTimer));
			pLed->bLedBlinkInProgress = false;
		}
		SwLedOff(dev, pLed);
		break;

	case LED_CTL_POWER_ON:
		pLed->CurrLedState = LED_ON;
		pLed->bLedBlinkInProgress = false;
		SwLedOn(dev, pLed);
		break;
	default:
		break;
	}
}

void
SwLedBlink(
	PLED_8185			pLed
	)
{
	struct net_device* dev = pLed->pAdapter;
	struct r8180_priv *priv = ieee80211_priv(dev);
	bool bStopBlinking = false;

	if((priv->eRFPowerState == eRfOff) && (priv->RfOffReason>RF_CHANGE_BY_IPS))
	{
		SwLedOff(dev, pLed);
		mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_ON_INTERVAL));
		return;
	}

	if( pLed->BlinkingLedState == LED_ON ) 
	{
		SwLedOn(dev, pLed);
	}	
	else 
	{
		SwLedOff(dev, pLed);
	}

	pLed->BlinkTimes--;
	switch(pLed->CurrLedState)
	{
	case LED_BLINK_NORMAL: 
		if(pLed->BlinkTimes == 0)
		{
			bStopBlinking = true;
		}
		break;

	case LED_SCAN_BLINK:
		if((((priv->ieee80211->state == IEEE80211_ASSOCIATING) || 
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATED) ||
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_RETRY) ||
		    	(priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATING)) || 
		        (priv->ieee80211->iw_mode == IW_MODE_ADHOC)) &&  
			(priv->ieee80211->scanning == false) && 
			(pLed->BlinkTimes % 2 == 0)) 
		{
			bStopBlinking = true;
		}
		break;

	case LED_NO_LINK_BLINK:
		if((priv->ieee80211->state == IEEE80211_ASSOCIATING) || 
		    (priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATED) ||
		    (priv->ieee80211->state == IEEE80211_ASSOCIATING_RETRY) ||
		    (priv->ieee80211->state == IEEE80211_ASSOCIATING_AUTHENTICATING))
		{
			bStopBlinking = true;
		}
		else if((priv->ieee80211->iw_mode == IW_MODE_ADHOC) && 
			    (priv->ieee80211->state == IEEE80211_LINKED))
		{
			bStopBlinking = true;
		}

		break;

	default:
		bStopBlinking = true;
		break;
	}

	if(bStopBlinking)
	{
		if( priv->eRFPowerState != eRfOn )
		{
			SwLedOff(dev, pLed);
		}
		else if(priv->ieee80211->state == IEEE80211_LINKED && pLed->bLedOn == false)
		{
			SwLedOn(dev, pLed);
		}
		else if(priv->ieee80211->state != IEEE80211_LINKED &&  pLed->bLedOn == true)
		{
			SwLedOff(dev, pLed);
		}

		pLed->BlinkTimes = 0;
		pLed->bLedBlinkInProgress = false;	
	}
	else
	{
		if( pLed->BlinkingLedState == LED_ON ) 
			pLed->BlinkingLedState = LED_OFF;
		else 
			pLed->BlinkingLedState = LED_ON;

		switch( pLed->CurrLedState )
		{
		case LED_BLINK_NORMAL:
			mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_BLINK_NORMAL_INTERVAL));
			break;

		case LED_BLINK_SLOWLY:
			mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_BLINK_SLOWLY_INTERVAL));
			break;

		case LED_SCAN_BLINK:
		case LED_NO_LINK_BLINK:
			if( pLed->bLedOn )
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_ON_INTERVAL));
			else
				mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_CM2_BLINK_OFF_INTERVAL));
			break;

		default:
			mod_timer(&(pLed->BlinkTimer), jiffies + MSECS(LED_BLINK_SLOWLY_INTERVAL));
			break;
		}		
	}
}


#ifdef NDIS50_MINIPORT
void
LedBlinkTimerStallCheck(
	struct net_device *dev
)
{	
}


void
DoLedTimerStallCheck(
	PLED_8185		pLed	
)
{	
}
#endif 

#endif 
